@@ -1,17 +1,9 @@
 from __future__ import annotations
 
+from collections.abc import Mapping
 from functools import partial
 from operator import methodcaller
-from typing import (
-    TYPE_CHECKING,
-    Any,
-    Callable,
-    Generic,
-    Literal,
-    Mapping,
-    Protocol,
-    Sequence,
-)
+from typing import TYPE_CHECKING, Any, Callable, Generic, Literal, Protocol
 
 from narwhals._compliant.any_namespace import (
     CatNamespace,
@@ -38,10 +30,9 @@ from narwhals._compliant.typing import (
 from narwhals._typing_compat import Protocol38, deprecated
 from narwhals._utils import _StoresCompliant, not_implemented
 from narwhals.dependencies import get_numpy, is_numpy_array
-from narwhals.dtypes import DType
 
 if TYPE_CHECKING:
-    from typing import Mapping
+    from collections.abc import Iterable, Mapping, Sequence
 
     from typing_extensions import Self, TypeIs
 
@@ -56,9 +47,9 @@ if TYPE_CHECKING:
     )
     from narwhals._expression_parsing import ExprKind, ExprMetadata
     from narwhals._utils import Implementation, Version, _FullContext
-    from narwhals.dtypes import DType
     from narwhals.typing import (
         FillNullStrategy,
+        IntoDType,
         NonNestedLiteral,
         NumericLiteral,
         RankMethod,
@@ -117,7 +108,7 @@ class CompliantExpr(Protocol38[CompliantFrameT, CompliantSeriesOrNativeExprT_co]
     def all(self) -> Self: ...
     def any(self) -> Self: ...
     def alias(self, name: str) -> Self: ...
-    def cast(self, dtype: DType | type[DType]) -> Self: ...
+    def cast(self, dtype: IntoDType) -> Self: ...
     def count(self) -> Self: ...
     def min(self) -> Self: ...
     def max(self) -> Self: ...
@@ -128,6 +119,7 @@ class CompliantExpr(Protocol38[CompliantFrameT, CompliantSeriesOrNativeExprT_co]
     def sum(self) -> Self: ...
     def median(self) -> Self: ...
     def skew(self) -> Self: ...
+    def kurtosis(self) -> Self: ...
     def std(self, *, ddof: int) -> Self: ...
     def var(self, *, ddof: int) -> Self: ...
     def n_unique(self) -> Self: ...
@@ -140,6 +132,8 @@ class CompliantExpr(Protocol38[CompliantFrameT, CompliantSeriesOrNativeExprT_co]
         limit: int | None,
     ) -> Self: ...
     def diff(self) -> Self: ...
+    def exp(self) -> Self: ...
+    def sqrt(self) -> Self: ...
     def unique(self) -> Self: ...
     def len(self) -> Self: ...
     def log(self, base: float) -> Self: ...
@@ -166,7 +160,7 @@ class CompliantExpr(Protocol38[CompliantFrameT, CompliantSeriesOrNativeExprT_co]
         old: Sequence[Any] | Mapping[Any, Any],
         new: Sequence[Any],
         *,
-        return_dtype: DType | type[DType] | None,
+        return_dtype: IntoDType | None,
     ) -> Self: ...
     def over(self, partition_by: Sequence[str], order_by: Sequence[str]) -> Self: ...
     def sample(
@@ -183,7 +177,7 @@ class CompliantExpr(Protocol38[CompliantFrameT, CompliantSeriesOrNativeExprT_co]
     def map_batches(
         self,
         function: Callable[[CompliantSeries[Any]], CompliantExpr[Any, Any]],
-        return_dtype: DType | type[DType] | None,
+        return_dtype: IntoDType | None,
     ) -> Self: ...
 
     def clip(
@@ -260,17 +254,17 @@ class CompliantExpr(Protocol38[CompliantFrameT, CompliantSeriesOrNativeExprT_co]
         return alias(names) if (alias := self._alias_output_names) else names
 
     @property
-    def str(self) -> Any: ...
+    def str(self) -> StringNamespace[Self]: ...
     @property
-    def name(self) -> Any: ...
+    def name(self) -> NameNamespace[Self]: ...
     @property
-    def dt(self) -> Any: ...
+    def dt(self) -> DateTimeNamespace[Self]: ...
     @property
-    def cat(self) -> Any: ...
+    def cat(self) -> CatNamespace[Self]: ...
     @property
-    def list(self) -> Any: ...
+    def list(self) -> ListNamespace[Self]: ...
     @property
-    def struct(self) -> Any: ...
+    def struct(self) -> StructNamespace[Self]: ...
 
 
 class DepthTrackingExpr(
@@ -514,7 +508,7 @@ class EagerExpr(
             scalar_kwargs=self._scalar_kwargs,
         )
 
-    def cast(self, dtype: DType | type[DType]) -> Self:
+    def cast(self, dtype: IntoDType) -> Self:
         return self._reuse_series("cast", dtype=dtype)
 
     def __eq__(self, other: Self | Any) -> Self:  # type: ignore[override]
@@ -613,6 +607,9 @@ class EagerExpr(
     def skew(self) -> Self:
         return self._reuse_series("skew", returns_scalar=True)
 
+    def kurtosis(self) -> Self:
+        return self._reuse_series("kurtosis", returns_scalar=True)
+
     def any(self) -> Self:
         return self._reuse_series("any", returns_scalar=True)
 
@@ -666,7 +663,7 @@ class EagerExpr(
 
     def filter(self, *predicates: Self) -> Self:
         plx = self.__narwhals_namespace__()
-        predicate = plx.all_horizontal(*predicates)
+        predicate = plx.all_horizontal(*predicates, ignore_nulls=False)
         return self._reuse_series("filter", predicate=predicate)
 
     def drop_nulls(self) -> Self:
@@ -677,7 +674,7 @@ class EagerExpr(
         old: Sequence[Any] | Mapping[Any, Any],
         new: Sequence[Any],
         *,
-        return_dtype: DType | type[DType] | None,
+        return_dtype: IntoDType | None,
     ) -> Self:
         return self._reuse_series(
             "replace_strict", old=old, new=new, return_dtype=return_dtype
@@ -804,7 +801,7 @@ class EagerExpr(
         )
 
     def map_batches(
-        self, function: Callable[[Any], Any], return_dtype: DType | type[DType] | None
+        self, function: Callable[[Any], Any], return_dtype: IntoDType | None
     ) -> Self:
         def func(df: EagerDataFrameT) -> Sequence[EagerSeriesT]:
             input_series_list = self(df)
@@ -874,7 +871,7 @@ class LazyExpr(
     ewm_mean: not_implemented = not_implemented()
     gather_every: not_implemented = not_implemented()
     replace_strict: not_implemented = not_implemented()
-    cat: not_implemented = not_implemented()  # pyright: ignore[reportAssignmentType]
+    cat: not_implemented = not_implemented()  # type: ignore[assignment]
 
     @property
     def window_function(self) -> WindowFunction[CompliantLazyFrameT, NativeExprT]: ...
@@ -897,10 +894,79 @@ class LazyExpr(
     @classmethod
     def _alias_native(cls, expr: NativeExprT, name: str, /) -> NativeExprT: ...
 
+    @classmethod
+    def _from_elementwise_horizontal_op(
+        cls, func: Callable[[Iterable[NativeExprT]], NativeExprT], *exprs: Self
+    ) -> Self: ...
+
     @property
     def name(self) -> LazyExprNameNamespace[Self]:
         return LazyExprNameNamespace(self)
 
+    def _with_binary(self, op: Callable[..., NativeExprT], other: Self | Any) -> Self: ...
+
+    def __eq__(self, other: Self) -> Self:  # type: ignore[override]
+        return self._with_binary(lambda expr, other: expr.__eq__(other), other)
+
+    def __ne__(self, other: Self) -> Self:  # type: ignore[override]
+        return self._with_binary(lambda expr, other: expr.__ne__(other), other)
+
+    def __add__(self, other: Self) -> Self:
+        return self._with_binary(lambda expr, other: expr.__add__(other), other)
+
+    def __sub__(self, other: Self) -> Self:
+        return self._with_binary(lambda expr, other: expr.__sub__(other), other)
+
+    def __rsub__(self, other: Self) -> Self:
+        return self._with_binary(lambda expr, other: other - expr, other).alias("literal")
+
+    def __mul__(self, other: Self) -> Self:
+        return self._with_binary(lambda expr, other: expr.__mul__(other), other)
+
+    def __truediv__(self, other: Self) -> Self:
+        return self._with_binary(lambda expr, other: expr.__truediv__(other), other)
+
+    def __rtruediv__(self, other: Self) -> Self:
+        return self._with_binary(lambda expr, other: other / expr, other).alias("literal")
+
+    def __floordiv__(self, other: Self) -> Self:
+        return self._with_binary(lambda expr, other: expr.__floordiv__(other), other)
+
+    def __rfloordiv__(self, other: Self) -> Self:
+        return self._with_binary(lambda expr, other: other // expr, other).alias(
+            "literal"
+        )
+
+    def __pow__(self, other: Self) -> Self:
+        return self._with_binary(lambda expr, other: expr.__pow__(other), other)
+
+    def __rpow__(self, other: Self) -> Self:
+        return self._with_binary(lambda expr, other: other**expr, other).alias("literal")
+
+    def __mod__(self, other: Self) -> Self:
+        return self._with_binary(lambda expr, other: expr.__mod__(other), other)
+
+    def __rmod__(self, other: Self) -> Self:
+        return self._with_binary(lambda expr, other: other % expr, other).alias("literal")
+
+    def __ge__(self, other: Self) -> Self:
+        return self._with_binary(lambda expr, other: expr.__ge__(other), other)
+
+    def __gt__(self, other: Self) -> Self:
+        return self._with_binary(lambda expr, other: expr.__gt__(other), other)
+
+    def __le__(self, other: Self) -> Self:
+        return self._with_binary(lambda expr, other: expr.__le__(other), other)
+
+    def __lt__(self, other: Self) -> Self:
+        return self._with_binary(lambda expr, other: expr.__lt__(other), other)
+
+    def __and__(self, other: Self) -> Self:
+        return self._with_binary(lambda expr, other: expr.__and__(other), other)
+
+    def __or__(self, other: Self) -> Self:
+        return self._with_binary(lambda expr, other: expr.__or__(other), other)
+
 
 class _ExprNamespace(  # type: ignore[misc]
     _StoresCompliant[CompliantExprT_co], Protocol[CompliantExprT_co]
@@ -1124,12 +1190,18 @@ class EagerExprStringNamespace(
     def to_datetime(self, format: str | None) -> EagerExprT:
         return self.compliant._reuse_series_namespace("str", "to_datetime", format=format)
 
+    def to_date(self, format: str | None) -> EagerExprT:
+        return self.compliant._reuse_series_namespace("str", "to_date", format=format)
+
     def to_lowercase(self) -> EagerExprT:
         return self.compliant._reuse_series_namespace("str", "to_lowercase")
 
     def to_uppercase(self) -> EagerExprT:
         return self.compliant._reuse_series_namespace("str", "to_uppercase")
 
+    def zfill(self, width: int) -> EagerExprT:
+        return self.compliant._reuse_series_namespace("str", "zfill", width=width)
+
 
 class EagerExprStructNamespace(
     EagerExprNamespace[EagerExprT], StructNamespace[EagerExprT], Generic[EagerExprT]
