@@ -2,21 +2,16 @@ from __future__ import annotations
 
 import operator
 from functools import partial
-from typing import (
-    TYPE_CHECKING,
-    Any,
-    Callable,
-    Iterator,
-    Literal,
-    Sequence,
-    TypeVar,
-    cast,
-)
+from typing import TYPE_CHECKING, Any, Callable, Literal, TypeVar, cast
 
 import ibis
 
 from narwhals._compliant import LazyExpr
 from narwhals._compliant.window import WindowInputs
+from narwhals._expression_parsing import (
+    combine_alias_output_names,
+    combine_evaluate_output_names,
+)
 from narwhals._ibis.expr_dt import IbisExprDateTimeNamespace
 from narwhals._ibis.expr_list import IbisExprListNamespace
 from narwhals._ibis.expr_str import IbisExprStringNamespace
@@ -25,6 +20,8 @@ from narwhals._ibis.utils import is_floating, lit, narwhals_to_native_dtype
 from narwhals._utils import Implementation, not_implemented
 
 if TYPE_CHECKING:
+    from collections.abc import Iterable, Iterator, Sequence
+
     import ibis.expr.types as ir
     from typing_extensions import Self
 
@@ -210,6 +207,23 @@ class IbisExpr(LazyExpr["IbisLazyFrame", "ir.Column"]):
             version=context._version,
         )
 
+    @classmethod
+    def _from_elementwise_horizontal_op(
+        cls, func: Callable[[Iterable[ir.Value]], ir.Value], *exprs: Self
+    ) -> Self:
+        def call(df: IbisLazyFrame) -> list[ir.Value]:
+            cols = (col for _expr in exprs for col in _expr(df))
+            return [func(cols)]
+
+        context = exprs[0]
+        return cls(
+            call=call,
+            evaluate_output_names=combine_evaluate_output_names(*exprs),
+            alias_output_names=combine_alias_output_names(*exprs),
+            backend_version=context._backend_version,
+            version=context._version,
+        )
+
     def _with_callable(
         self, call: Callable[..., ir.Value], /, **expressifiable_args: Self | Any
     ) -> Self:
@@ -225,7 +239,7 @@ class IbisExpr(LazyExpr["IbisLazyFrame", "ir.Column"]):
         def func(df: IbisLazyFrame) -> list[ir.Value]:
             native_series_list = self(df)
             other_native_series = {
-                key: df._evaluate_expr(value) if self._is_expr(value) else value
+                key: df._evaluate_expr(value) if self._is_expr(value) else lit(value)
                 for key, value in expressifiable_args.items()
             }
             return [
@@ -241,6 +255,9 @@ class IbisExpr(LazyExpr["IbisLazyFrame", "ir.Column"]):
             version=self._version,
         )
 
+    def _with_binary(self, op: Callable[..., ir.Value], other: Self | Any) -> Self:
+        return self._with_callable(op, other=other)
+
     def _with_alias_output_names(self, func: AliasNames | None, /) -> Self:
         return type(self)(
             self._call,
@@ -265,78 +282,6 @@ class IbisExpr(LazyExpr["IbisLazyFrame", "ir.Column"]):
     def _alias_native(cls, expr: ExprT, name: str, /) -> ExprT:
         return cast("ExprT", expr.name(name))
 
-    def __and__(self, other: IbisExpr) -> Self:
-        return self._with_callable(lambda expr, other: expr & other, other=other)
-
-    def __or__(self, other: IbisExpr) -> Self:
-        return self._with_callable(lambda expr, other: expr | other, other=other)
-
-    def __add__(self, other: IbisExpr) -> Self:
-        return self._with_callable(lambda expr, other: expr + other, other=other)
-
-    def __truediv__(self, other: IbisExpr) -> Self:
-        return self._with_callable(lambda expr, other: expr / other, other=other)
-
-    def __rtruediv__(self, other: IbisExpr) -> Self:
-        return self._with_callable(
-            lambda expr, other: expr.__rtruediv__(other), other=other
-        ).alias("literal")
-
-    def __floordiv__(self, other: IbisExpr) -> Self:
-        return self._with_callable(
-            lambda expr, other: expr.__floordiv__(other), other=other
-        )
-
-    def __rfloordiv__(self, other: IbisExpr) -> Self:
-        return self._with_callable(
-            lambda expr, other: expr.__rfloordiv__(other), other=other
-        ).alias("literal")
-
-    def __mod__(self, other: IbisExpr) -> Self:
-        return self._with_callable(lambda expr, other: expr.__mod__(other), other=other)
-
-    def __rmod__(self, other: IbisExpr) -> Self:
-        return self._with_callable(
-            lambda expr, other: expr.__rmod__(other), other=other
-        ).alias("literal")
-
-    def __sub__(self, other: IbisExpr) -> Self:
-        return self._with_callable(lambda expr, other: expr - other, other=other)
-
-    def __rsub__(self, other: IbisExpr) -> Self:
-        return self._with_callable(
-            lambda expr, other: expr.__rsub__(other), other=other
-        ).alias("literal")
-
-    def __mul__(self, other: IbisExpr) -> Self:
-        return self._with_callable(lambda expr, other: expr * other, other=other)
-
-    def __pow__(self, other: IbisExpr) -> Self:
-        return self._with_callable(lambda expr, other: expr**other, other=other)
-
-    def __rpow__(self, other: IbisExpr) -> Self:
-        return self._with_callable(
-            lambda expr, other: expr.__rpow__(other), other=other
-        ).alias("literal")
-
-    def __lt__(self, other: IbisExpr) -> Self:
-        return self._with_callable(lambda expr, other: expr < other, other=other)
-
-    def __gt__(self, other: IbisExpr) -> Self:
-        return self._with_callable(lambda expr, other: expr > other, other=other)
-
-    def __le__(self, other: IbisExpr) -> Self:
-        return self._with_callable(lambda expr, other: expr <= other, other=other)
-
-    def __ge__(self, other: IbisExpr) -> Self:
-        return self._with_callable(lambda expr, other: expr >= other, other=other)
-
-    def __eq__(self, other: IbisExpr) -> Self:  # type: ignore[override]
-        return self._with_callable(lambda expr, other: expr == other, other=other)
-
-    def __ne__(self, other: IbisExpr) -> Self:  # type: ignore[override]
-        return self._with_callable(lambda expr, other: expr != other, other=other)
-
     def __invert__(self) -> Self:
         invert = cast("Callable[..., ir.Value]", operator.invert)
         return self._with_callable(invert)
@@ -365,9 +310,15 @@ class IbisExpr(LazyExpr["IbisLazyFrame", "ir.Column"]):
         return self._with_callable(lambda expr: expr.quantile(quantile))
 
     def clip(self, lower_bound: Any, upper_bound: Any) -> Self:
-        def _clip(expr: ir.NumericValue, lower: Any, upper: Any) -> ir.NumericValue:
+        def _clip(
+            expr: ir.NumericValue, lower: Any | None = None, upper: Any | None = None
+        ) -> ir.NumericValue:
             return expr.clip(lower=lower, upper=upper)
 
+        if lower_bound is None:
+            return self._with_callable(_clip, upper=upper_bound)
+        if upper_bound is None:
+            return self._with_callable(_clip, lower=lower_bound)
         return self._with_callable(_clip, lower=lower_bound, upper=upper_bound)
 
     def sum(self) -> Self:
@@ -673,6 +624,12 @@ class IbisExpr(LazyExpr["IbisLazyFrame", "ir.Column"]):
 
         return self._with_callable(_exp)
 
+    def sqrt(self) -> Self:
+        def _sqrt(expr: ir.NumericColumn) -> ir.Value:
+            return ibis.cases((expr < lit(0), lit(float("nan"))), else_=expr.sqrt())
+
+        return self._with_callable(_sqrt)
+
     @property
     def str(self) -> IbisExprStringNamespace:
         return IbisExprStringNamespace(self)
@@ -695,4 +652,5 @@ class IbisExpr(LazyExpr["IbisLazyFrame", "ir.Column"]):
 
     # NOTE: https://github.com/ibis-project/ibis/issues/11176
     skew = not_implemented()
+    kurtosis = not_implemented()
     unique = not_implemented()
